//
//  SITIndication.h
//  SitumSDK
//
//  Created by A Barros on 21/2/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>


/**
 The type of action the user should perform in order to arrive to the destination.
 */
typedef NS_ENUM(NSInteger, kSITIndicationActions) {
    /// Calculating directions...
    kSITCalculating,
    /// invalid action
    kSITInvalidAction,
    /// the user needs to turn
    kSITTurn,
    /// the user needs to keep the same direction
    kSITGoAhead,
    /// the user needs to change floor
    kSITChangeFloor,
    /// the user has arrived to the destination
    kSITEnd

};


/**
 The type of turn (if any) the user should perform in order to arrive to the destination.
 */
typedef NS_ENUM(NSInteger, kSITIndicationOrientation) {
    /// invalid orientation.
    kSITInvalidOrientation,
    /// keep the same direction.
    kSITStraight,
    /// turn slightly to the right.
    kSITVeerRight,
    /// turn right.
    kSITRight,
    /// turn heavily to right.
    kSITSharpRight,
    /// turn slightly to the left.
    kSITVeerLeft,
    /// turn left.
    kSITLeft,
    /// turn heavily to the left.
    kSITSharpLeft,
    /// turn around.
    kSITBackward,
};


/**
 Represents the instruction that a user should follow when on a SITRouteStep to continue the route.
 */
@interface SITIndication : NSObject

#pragma mark - Properties

/**
 The index of the step where the indication starts (positive integer).
 */
@property (nonatomic, readonly) NSInteger originStepIndex;


/**
 The index of the step where the indication ends (positive integer).
 */
@property (nonatomic, readonly) NSInteger destinationStepIndex;

/**
 Distance between origin and destination (in meters).
 */
@property (nonatomic, readonly) float horizontalDistance;

/**
 Number of levels between the origin and destination.
 @note value representing the number of levels to go upstairs (positive values) or downstairs (negative values).
 */
@property (nonatomic, readonly) float verticalDistance;

/**
 Check if the user should change the level in order to arrive to destination.
 */
@property (nonatomic, readonly) BOOL needLevelChange;

/**
 Destination level of the step
 */
@property (nonatomic, readwrite) NSNumber* _Nullable nextLevel;

/**
 The angle between the previous destination and the line formed by the origin and destination of the current step (in radians).
 */
@property (nonatomic, readonly) float orientationChange;


/**
 The type of action the user should perform in order to arrive to the destination.
 @note one of kSITIndicationActions.
 */
@property (nonatomic, readonly) kSITIndicationActions action;

/**
 The type of turn (if any) the user should perform in order to arrive to the destination.
 @note one of kSITIndicationOrientation.
 */
@property (nonatomic, readonly) kSITIndicationOrientation orientation;

#pragma mark - Methods

/**
 Human readable text. Language will be set based on the language selected by the user in the system's preferences.
 Only english and spanish are supported, if an unknown locale is set, english will be used as default.

 @return string that describes the indication in a more human friendly way
 */
- (NSString * _Nonnull)humanReadableMessage;

/**
 Human readable text in the language selected by the locale parameter.
 Only english and spanish are supported, if an unknown locale is passed, english will be used as default.
 
 @return string that describes the indication in a more human friendly way
 */
- (NSString * _Nonnull) humanReadableMessageWithLocale: (NSLocale* _Nonnull) locale;

/**
 
 Creates an indication with calculating action.
 
 @return an indication
 */
+ (SITIndication* _Nonnull) indicationWithCalculatingAction;

/**
 
 Creates an indication with end action.
 
 @return an indication
 */
+ (SITIndication* _Nonnull) indicationWithEndAction;

#pragma mark - Private methods

/*
 Private method.
 Initialize a SITIndication object.
 
 @param stepIndex index of the origin SITRouteStep
 @param destinationStepIndex index of the destination of a SITRouteStep
 @param orientationChange the angle between the virtual line of (in radians)
 @param horizontalDistance the distance between the origin and destination SITRouteStep
 @param verticalDistance the number of levels that needs to go up (positive value) or down (negative value) in order to arrive to the destination.
 @param nextLevel Destination level
 @return an indication
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (instancetype _Nonnull)initWithStepIndex:(NSInteger)stepIndex
                      destinationStepIndex:(NSInteger)destinationStepIndex
                         orientationChange:(float)orientationChange
                        horizontalDistance:(float)horizontalDistance
                          verticalDistance:(float)verticalDistance
                                 nextLevel:(NSNumber* _Nonnull)nextLevel;

/*
 Private method.
 Initialize a SITIndication object.
 
 @param stepIndex index of the origin SITRouteStep
 @param destinationStepIndex index of the destination of a SITRouteStep
 @param orientationChange the angle between the virtual line of (in radians)
 @param horizontalDistance the distance between the origin and destination SITRouteStep
 @param verticalDistance the number of levels that needs to go up (positive value) or down (negative value) in order to arrive to the destination.
 @return an indication
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (instancetype _Nonnull)initWithStepIndex:(NSInteger)stepIndex
                      destinationStepIndex:(NSInteger)destinationStepIndex
                         orientationChange:(float)orientationChange
                        horizontalDistance:(float)horizontalDistance
                          verticalDistance:(float)verticalDistance;

/*
 Private method.
 
 @param originStepIndex index of the origin SITRouteStep.
 @param destinationStepIndex index of the destination SITRouteStep.
 @param action the type of action the user should perform in order to arrive to the destination.
 @param horizontalDistance distance between origin and destination (in meters).
 @param orientation The type of turn (if any) the user should perform in order to arrive to the destination.
 @param orientationChange The angle between the previous destination and the line formed by the origin and destination of the current step (in radians).
 @param verticalDistance Number of levels between the origin and destination.
 @param nextLevel Destination level
 @return an indication
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (instancetype _Nonnull)initWithOriginStepIndex:(NSInteger)originStepIndex
                            destinationStepIndex:(NSInteger)destinationStepIndex
                                          action:(kSITIndicationActions)action
                              horizontalDistance:(float)horizontalDistance
                                     orientation:(kSITIndicationOrientation)orientation
                               orientationChange:(float)orientationChange
                                verticalDistance:(float)verticalDistance
                                       nextLevel:(NSNumber* _Nonnull)nextLevel;

/*
 Private method.
 
 @param originStepIndex index of the origin SITRouteStep.
 @param destinationStepIndex index of the destination SITRouteStep.
 @param action the type of action the user should perform in order to arrive to the destination.
 @param horizontalDistance distance between origin and destination (in meters).
 @param orientation The type of turn (if any) the user should perform in order to arrive to the destination.
 @param orientationChange The angle between the previous destination and the line formed by the origin and destination of the current step (in radians).
 @param verticalDistance Number of levels between the origin and destination.
 @return an indication
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (instancetype _Nonnull)initWithOriginStepIndex:(NSInteger)originStepIndex
                            destinationStepIndex:(NSInteger)destinationStepIndex
                                          action:(kSITIndicationActions)action
                              horizontalDistance:(float)horizontalDistance
                                     orientation:(kSITIndicationOrientation)orientation
                               orientationChange:(float)orientationChange
                                verticalDistance:(float)verticalDistance;

/*
 Private method to check if two indication can be chunked.

 @param indication the indication that needs to be chuncked.
 @return BOOL value that indicates if the indication can be chunked (YES) or not (NO)
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (BOOL)isChunckableWithIndication:(SITIndication * _Nonnull)indication;

/*
 Private method to compress two indications into one.

 @param indication the indication to be chunked.
 @return a compress indication.
 @note You should not use this method directly. Instead you should use the indications provided by SITDirectionsManager or SITNavigationManager.
 */
/// :nodoc:
- (SITIndication * _Nonnull)chuckWithIndication:(SITIndication * _Nonnull)indication;


@end
